@@ -1,6 +1,5 @@
 # **************************************************************************
-# *
-# * Authors:    Mohamad Harastani            (mohamad.harastani@upmc.fr)
+# * Authors:    Mohamad Harastani            (mohamad.harastani@igbmc.fr)
 # *             Slavica Jonic                (slavica.jonic@upmc.fr)
 # *
 # * This program is free software; you can redistribute it and/or modify
@@ -20,21 +19,13 @@
 # *
 # *  All comments concerning this program package may be sent to the
 # *  e-mail address 'scipion@cnb.csic.es'
-# *
 # **************************************************************************
+
 from continuousflex.protocols.data import PathData
-
-"""
-This module implement the wrappers around Xmipp CL2D protocol
-visualization program.
-"""
-
 from os.path import basename, join, exists, isfile
 import numpy as np
-
-from pyworkflow.utils.path import cleanPath, makePath, cleanPattern
+from pyworkflow.utils.path import cleanPath, makePath
 from pyworkflow.viewer import (ProtocolViewer, DESKTOP_TKINTER, WEB_DJANGO)
-
 from pyworkflow.protocol.params import StringParam, LabelParam
 from pwem.objects import SetOfParticles
 from pwem.viewers import VmdView
@@ -272,14 +263,10 @@ class FlexDimredNMAVolViewer(ProtocolViewer):
         cleanPath(fnSqlite)
         partSet = SetOfParticles(filename=fnSqlite)
         partSet.copyInfo(inputSet)
-        first = True
         for point in self.getData():
             if point.getState() == Point.SELECTED:
                 particle = inputSet[point.getId()]
                 partSet.append(particle)
-                if first:
-                    flag = particle._xmipp_angleY.get()
-                    first = False
         partSet.write()
         partSet.close()
 
@@ -291,7 +278,6 @@ class FlexDimredNMAVolViewer(ProtocolViewer):
             newProt.setObjLabel(clusterName)
         newProt.inputNmaDimred.set(prot)
         newProt.sqliteFile.set(fnSqlite)
-        newProt.angleYflag.set(flag)
         project.launchProtocol(newProt)
         project.getRunsGraph()
 
